#!/usr/bin/env python

__version__ = '$Revision: 5061 $'.split()[1]
__date__ = '$Date: 2006-11-29 17:34:11 -0500 (Wed, 29 Nov 2006) $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./xmlbinmsg.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector
import aisstring

import binary
#def waterlevelEncode(*aDict, **params):
def encode(params, validate=False, **moreParams):
	'''Serializer for the waterlevel binary message
	
	Keywords and types:

	  - dac:  uint
	  - unavail_uint:  uint
	  - uint:  uint
	FIX: generate CORRECT doctest string that does all the defaults

	The default message:

	>>> print encode()
	1011011101100

	@param aDict: for passing in a dictionary of keyword and values.
	@param params: keyword dictionary or if a dict is passed, it will use that dict
	@return: bitvector
	@note: only use one of aDict or params
	@bug: FIX: have moreParams add to and override params
	'''

	bvList = []
	

	### FIELD: dac (type=uint)   REQUIRED CONSTANT FIELD
	bvList.append(binary.setBitVectorSize(BitVector(intVal=366),16))

	### FIELD: unavail_uint (type=uint)
	if 'unavail_uint' in params: bvList.append(binary.setBitVectorSize(BitVector(intVal=params['unavail_uint']),2))
	else: bvList.append(binary.setBitVectorSize(BitVector(intVal=3),2))

	### FIELD: anUInt (type=uint)
	#if 'anUInt' in params:
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['anUInt']),2))
	#else: bvList.append(BitVector(size=2))

	### FIELD: anInt (type=int)
	#if 'int' in params: 
	bvList.append(binary.bvFromSignedInt(int(params['anInt']),4))
        #else: assert False # FIX: throw exception when not given

	### FIELD: aBool (type=bool)
	if params['aBool']: bvList.append(BitVector(bitstring="1"))
	else: bvList.append(BitVector(bitstring="0"))

	if validate and len(params['aStr']) != 5: raise ValueError
	bvList.append(aisstring.encode(params['aStr'],5*6))

	# 16 bits
	# scaleValue = 10
	#tmpVal = int(params['anUDecimal']/Decimal('.1'))
	#print 'an unsigned decimal tmpVal: ', params['anUDecimal'], '->',tmpVal
	#bvList.append(binary.setBitVectorSize(BitVector(intVal=tmpVal),16))
	#del tmpVal
	bvList.append(binary.setBitVectorSize(BitVector(intVal=int(params['anUDecimal']/Decimal('.1'))),16))

	#tmpVal = int(params['aDecimal']/Decimal('10'))
	#print 'a decimal tmpVal: ', params['aDecimal'], '->',tmpVal
	#bvList.append(binary.bvFromSignedInt(tmpVal,16))
	bvList.append(binary.bvFromSignedInt(int(params['aDecimal']/Decimal('10')),16))

	# FIELD: aFloat (type=float)
	bvList.append(binary.float2bitvec(params['aFloat']))

	return binary.joinBV(bvList)


######################################################################

def decode(bv,validate=False):
	'''Deserialize a binary water levelmessage

	@param bv: bitvector containing the message portion of the assembled transmission
	@param validate: set to true to check parsed values.  If false, constant fields will not be parsed.
	@return: dictory of fields and values
	'''

	r={} # Results
	r['dac'] = 366 # Constand value
	if int(bv[0:16]) != 366: raise ValueError

	print 'unavail: ',bv[16:16+2],int(bv[16:16+2])
	r['unavail_uint'] = int(bv[16:16+2])
	# FIX: what to do if this is set to the unavailable value?  Prob another dict value of a bool with _available

	r['anUInt'] = int(bv[18:18+2])
	r['anInt'] = binary.signedIntFromBV(bv[20:20+4])
	print 'bool:',bv[24:]
	r['aBool'] = bool(int(bv[24]))
	
	r['aStr'] = aisstring.decode(bv[25:25+5*6])

	r['anUDecimal'] = int(bv[55:55+16])*Decimal('.1')
	r['aDecimal'] = binary.signedIntFromBV(bv[55+16:55+32])*Decimal('10')

	base = 55+32
	r['aFloat'] = binary.bitvec2float(bv[base:base+32])
	base += 32

	return r

######################################################################
# UNIT TESTING
######################################################################
import unittest

class TestEncode(unittest.TestCase):
    '''Unit testing of the Template class.  
    Make lots of testFoo methods to exercise the class.
    '''
    def testEncodeAllPresent(self):
	'''Just encode
	'''
	params = {}
	params['unavail_uint']=1
	params['anUInt']=2
	params['anInt']=-3
	params['aBool']=True
	params['anUDecimal']=Decimal('4.5')
	params['aDecimal']=-70000
	params['aStr']='12345'
	params['aFloat'] = -234.56
	print params
	bv = encode(params)
	print bv
	print 'decoding...'
	r = decode (bv,validate=True)
	print 'dict of decoded values:', r
	self.failUnless(r['dac']==366)
	self.failUnless(r['unavail_uint']==1)
	self.failUnless(r['anUInt']==2)
	self.failUnless(r['anInt']==-3)
	self.failUnless(r['aBool']==True)
	self.failUnless(r['anUDecimal']==Decimal('4.5'))
	self.failUnless(r['aDecimal']==Decimal('-70000'))
	print 'retrieved aFloat:',r['aFloat']
	self.failUnlessAlmostEqual(r['aFloat'],params['aFloat'],2)

	#bv = binary.setBitVectorSize(BitVector(intVal=366),16)
	#decode (bv,validate=True)
	#bv = BitVector(bitstring="01")
	#decode (bv,validate=True)
	#self.failUnless(1==1)


############################################################
if __name__=='__main__':
#    print encode()
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)

    #sys.exit('EARLY EXIT - FIX: remove')
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='Make the test output verbose')

    (options,args) = myparser.parse_args()
    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc


    if options.unittest:
	sys.argv = [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')

	unittest.main()
    
